#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yumexBase.py,v 1.30 2006/02/18 06:33:54 timlau Exp $

# Yum Extender Yum Classes 

import sys
import yumex.yumexConst as const

import os
import locale
import time # test purposes only
import pygtk
import gtk
import pango
import re

import rpm


import yum
import yum.Errors as Errors
import yumgui
import output
import callback
import rpmUtils
import fcntl
import fnmatch
import yum.repos
import traceback
import logging
import yum.logginglevels
import ConfigParser

from yum.packages import parsePackages, YumLocalPackage, YumInstalledPackage

from rpmUtils.miscutils import compareEVR
    


import urlgrabber

from urlgrabber.progress import *

from yumex.yumexi18n import _

from yumex.yumexCallbacks import *

import yumex.bestmirror as bestmirror
import yumex.fastestmirror as fastestmirror

class YumexError( yum.Errors.YumBaseError ):
   def __init__( self, msg ):
        yum.Errors.YumBaseError.__init__( self )
        self.msg = msg

class yumexBase( yumgui.Base, output.YumOutput ):
    """ This class does all the interfacing with yum """

    def __init__( self, yumexconf, logger, progress=None, loglevel=2 ):
        yumgui.Base.__init__( self, logger )
        self.yumex_logger = logging.getLogger( 'yumex.yumexBase' )   
        self.progress = progress
        #self.console = console
        self.last_error = None
        self.loglevel = loglevel
        # FIXME: get rid of the oldlogger stuff & use python logging
        self.oldlogger = logger
        self.yumexconf = yumexconf
        if self.yumexconf.debug:
            self.show_settings()
        self.downloadProgress = yumexDownloadProgress( self, progress )
            
                                
        self.repos_in_cache = yumexRepoCache( const.MIRROR_RECENT_LIMIT )
        md = const.MIRROR_DETECTION_TYPES[self.yumexconf.mirrordetection]
        self.log( 2, _( "Mirrordetection : %s" ) % md )
        # setup our depsolve progress callback
        dscb = DepSolveProgressCallBack( self.log, self.errorlog )
        self.dsCallback = dscb
        self.log( 2, _( "Yum Version : %s (%s)" ) % ( yum.__version__, sys.path[0] ) )
            
    def show_settings( self ):
        self.log( 2, _( "Current Settings :" ) )
        settings = str( self.yumexconf ).split( '\n' )
        for s in settings:
            if s.startswith( '[' ):
                pass
            else:
                self.log( 2, "    %s" % s )

    def cleanup( self ):
        """ Clean out datastructures, so setup can be called again. """
        # Kill old RepoStorage and make new one.
        self.repos = yum.repos.RepoStorage()    
        # Kill the current pkgSack    
        if self.pkgSack:
            self.pkgSack=None
        self.closeRpmDB()
        
    def reset_ts( self ):
        """  Reset Transaction Set and Setup it up again
        Used on Errors and User Cancels to reinit the yum data structures   """
        self.closeRpmDB()
        self.progresslog( _( "Setup Yum : Transaction Set" ) )
        self.doTsSetup()
        self.progresslog( _( "Setup Yum : RPM Db." ) )
        self.doRpmDBSetup()
        self.doUpdateSetup()
        
        
        

    def setup( self, reload=False ):
        try:
            self.progresslog( _( "Setup Yum : Config" ) )
            # Only use CORE Plugins
            if not self.yumexconf.noplugins:
                self.progresslog( _( "Setup Yum : Plugins" ) )
                self.doConfigSetup( plugin_types=( yum.plugins.TYPE_CORE, ), debuglevel=self.yumexconf.yumdebuglevel )
            else:
                self.progresslog( _( "Setup Yum : Plugins are disabled" ) )
                self.doConfigSetup( init_plugins=False, debuglevel=self.yumexconf.yumdebuglevel )
            # Lock Yum
            self.doLock( const.YUM_PID_FILE )
            self.setupFileLog()
            self.filelog = logging.getLogger( "yumex.filelogging" )
            #self.doFileLogSetup(os.geteuid(),self.conf.logfile)
            self.conf.assumeyes= True # Disable questions asked
            self.conf.uid = os.geteuid() # who are we
            self.conf.debuglevel = self.yumexconf.yumdebuglevel
            self.repos_in_cache.load()            
            if self.yumexconf.usecache:
                self.conf.cache = True
                self.log( 2, _( "Using yum cache" ) ) 
            self.repos.callback = CacheProgressCallback( self.log, self.errorlog, self.filelog, self, self.progress )
            self.setup_excludes()
            self.progresslog( _( "Setup Yum : Transaction Set" ) )
            self.doTsSetup()
            self.progresslog( _( "Setup Yum : RPM Db." ) )
            self.doRpmDBSetup()
            self.progresslog( _( "Setup Yum : Base setup completed" ) )
            return True
        except yum.Errors.RepoError, e:
            self.last_error = str( e )
            return False
        except yum.Errors.LockError, e:
            self.last_error = _( 'Yum is locked by another application' )
            return False

    def doLoggingSetup( self, debuglevel, errorlevel ):
        '''
        Perform logging related setup.

        @param debuglevel: Debug logging level to use.
        @param errorlevel: Error logging level to use.
        '''
        #logginglevels.doLoggingSetup(errorlevel, debuglevel)
        lvl = yum.logginglevels.logLevelFromDebugLevel( debuglevel )
        logging.getLogger( 'yum' ).setLevel( lvl )
        

    def doRepoSetup( self, thisrepo=None ):
        """grabs the repomd.xml for each enabled repository and sets up 
           the basics of the repository"""

        self.plugins.run( 'prereposetup' )
        
        if thisrepo is None:
            repos = self.repos.listEnabled()
        else:
            repos = self.repos.findRepos( thisrepo )

        if len( repos ) < 1:
            self.errorlog( 0, 'No Repositories Available to Set Up' )

        num = 1
        for repo in repos:
            # repo.setup(self.conf.cache)
            # This code is from setup in YumRepository (yum/yumRepos.py)
            try:
                if repo.repoXML:
                    continue
                repo.cache = self.conf.cache
                self.progresslog( _( "Loading Repositories Data : Init. %s repository" ) % repo.id )                    
                repo.baseurlSetup()
                
                #
                # Yumex mirror MirrorDectection stuff.
                #
                
                if not self.yumexconf.usecache:                
                    if not self.repos_in_cache.has_item( repo.id, self.yumexconf.mirrordetection ):
                        self.doMirrorDetectionSetup( repo )
                    else:
                        self.progresslog( _( "Reading mirror URLs from yumex cache" ) )
                        urls = self.repos_in_cache.get_urls( repo.id )
                        if urls != None:
                            repo.setAttribute( 'urls', urls )
                            # Only set failovermethod=priority if mirrordetection for repo
                            # is best or fastest
                            if self.yumexconf.mirrordetection != 'default':
                                repo.setAttribute( 'failovermethod', 'priority' )
                            repo.check()
                            repo.setupGrab()

                repo.dirSetup()
            except Errors.RepoError, e:
                raise

            try:
                repo._loadRepoXML( text=self )
            except Errors.RepoError, e:
                raise Errors.RepoError, ( 'Cannot open/read repomd.xml file for repository: %s' % repo )
            num += 1
            
            
            if self.repos.callback and len( repos ) > 0:
                self.repos.callback.progressbar( num, len( repos ), repo.id )
            
        self.plugins.run( 'postreposetup' )



    def doMirrorDetectionSetup( self, repo ):
        md = const.MIRROR_DETECTION_TYPES[self.yumexconf.mirrordetection]
        if self.yumexconf.mirrordetection != 'default':
            self.progresslog( _( "Determining %s mirrors for %s" ) % ( md, repo.id ) )
        repomirrors = []
        if self.yumexconf.mirrordetection == 'fastest':
            fastestmirror.verbose = False
            fastestmirror.hostfilepath = '/var/cache/yum/yumex-fastest-hosts.txt'
            fastestmirror.socket_timeout = 3
            repomirrors = fastestmirror.FastestMirror( repo.urls ).get_mirrorlist()
        elif self.yumexconf.mirrordetection == 'best':
            bestmirror.verbose = False
            bestmirror.hostfilepath = '/var/cache/yum/yumex-best-hosts.txt'
            bestmirror.socket_timeout = 3
            repomirrors = bestmirror.BestMirror( repo.id, repo.urls ).get_mirrorlist()
        else:
            repomirrors = []
        if len( repomirrors ) > 0:
            repo.setAttribute( 'urls', repomirrors )
            if self.yumexconf.debug:
                self.log( 2, _( "--> Mirrors Detected (Top 3)" ) )
                i=0
                for url in repomirrors:
                    self.log( 2, "---> %s" % url )
                    i += 1
                    if i == 3:
                        break
            repo.setAttribute( 'failovermethod', 'priority' )
            repo.check()
            repo.setupGrab()            
        else:
            self.log( 2, _( "Using default mirror detection" ) )
           
    def setup_excludes( self ):
        """ Append the yumex excludes to the yum exclude list """
        try:
            excludelist = self.conf.exclude
            for ex in self.yumexconf.exclude:
                excludelist.append( ex )
            self.conf.exclude = excludelist
        except yum.Errors.ConfigError, e:
            self.errorlog( 0, _( e ) )

    def failureReport( self, errobj ):
        """failure output for failovers from urlgrabber"""
        
        self.yumex_logger.error( _( 'Failure getting %s: ' ), errobj.url )
        lines = str( errobj.exception ).split( '\n' )
        #print lines
        for line in lines:
            self.yumex_logger.error( '  --> %s' % line )
        self.yumex_logger.error( 'Trying other mirror.' )
        raise errobj.exception

        
    def load_repo_data( self, reload=False ):
        self.progresslog( _( "Loading Repositories Data" ) )
        self.repos.setProgressBar( self.downloadProgress )
        # Setup failure callback
        freport = ( self.failureReport, (), {} )
        self.repos.setFailureCallback( freport )
        
        self.doTsSetup()
        try:
            
            self.doRepoSetup()    # Setup Repositories
            self.progresslog( _( "Setting Up Package Sack" ) )
            self.doSackSetup()    # Setup Package Sack.
            if self.yumexconf.filelists: 
                # Populate package with all info.
                self.progresslog( _( "Setting Up Package Sack : Filelists" ) )
                self.repos.populateSack(with='filelists')
                self.progresslog( _( "Setting Up Package Sack : Changelogs" ) )
                self.repos.populateSack(with='otherdata')
            self.progresslog( _( "Loading Repository Group Data" ) )
            self.doGroupSetup()
            self.progresslog( _( "Loading Repositories Data completed" ) )

            repos = self.repos.listEnabled()
            for repo in repos:
                if not self.repos_in_cache.has_item( repo.id, self.yumexconf.mirrordetection ):
                    self.repos_in_cache.add( repo.id )
                    self.repos_in_cache.set_urls( repo.id, repo.urls, self.yumexconf.mirrordetection )
            if self.repos_in_cache.updated: 
                self.repos_in_cache.save()
                self.log( 2, _( "Writing Mirrorcache to disk" ) )
                if self.yumexconf.debug:
                    reponames = ",".join( self.repos_in_cache.get_repo_names() )
                    self.log( 2, _( "--> Repositories saved: %s" ) % reponames )
                    
                    
                
            return True
        except yum.Errors.RepoError, e:
            self.last_error = str( e )
            return False  

        
    def doGroupSetup( self ):
        """ Get Group Info from repos"""
        try:
            yum.YumBase.doGroupSetup( self )            
        except yum.Errors.GroupsError, e:
            self.log( 2, str( e ) )
            
    def userconfirm( self ):
        return True
    
    def getGroupPackages( self, grp ):
        pkglist = []
        condList = []
        group = self.comps.return_group( grp )
        for item in group.conditional_packages.keys():
            req = group.conditional_packages[item]
            p, reqinst = self._get_pkg( req )
            if reqinst:
                condList.append( item )
        pkglist = group.mandatory_packages.keys()+group.default_packages.keys()+condList
        return pkglist
        
    def getGroupData( self, grp ):
        #print '\nGroup: %s' % group
        main = {}
        group = self.comps.return_group( grp )
        #print group.groupid
        
        if len( group.mandatory_packages.keys() ) > 0:
            pkgs = []
            for item in group.mandatory_packages.keys():
                 p, installed = self._get_pkg( item )
                 if len( p )>0: # Package Found
                     pkgs.append( ( p[0], installed ) )
            main['mandatory'] =  pkgs
        if len( group.default_packages.keys() ) > 0:
            pkgs = []
            for item in group.default_packages.keys():
                 p, installed = self._get_pkg( item )
                 if len( p )>0: # Package Found
                     pkgs.append( ( p[0], installed ) )
            main['default'] =  pkgs
        if len( group.optional_packages.keys() ) > 0:
            pkgs = []
            for item in group.optional_packages.keys():
                 p, installed = self._get_pkg( item )
                 if len( p )>0: # Package Found
                     pkgs.append( ( p[0], installed ) )
            main['optional'] =  pkgs
        if len( group.conditional_packages.keys() ) > 0:
            #print group.conditional_packages.keys()
            pkgs = []
            for item in group.conditional_packages.keys():
                 req = group.conditional_packages[item]
                 #print "%s requires %s" % (item,req)
                 p, reqinst = self._get_pkg( req )
                 if reqinst:
                     #print "  installed"
                     p, installed = self._get_pkg( item )
                     if len( p )>0: # Package Found
                         pkgs.append( ( p[0], installed ) )
                # else:
                 #    print " not installed"
            main['conditional'] =  pkgs
        return main
    
    def _get_pkg( self, name ):
        installed = self.isPkgInstalled( name=name )
        if installed:
            p = self.findPackages( inst=True, avail=False, name=name )
        else:
            p = self.findPackages( inst=False, best=True, name=name )
        return p, installed

    def _get_pkg_info( self, name ):
        p, installed = self._get_pkg( name )
        if installed:
            state = 1
        else:
            state = 0
        if len( p ) > 0:
            desc = p[0].returnSimple( 'summary' ).splitlines()[0]
            return [p[0].compactPrint(), state, desc]
        else:
            return [name, state, ""]
        
    def set_active_repos( self, repolist ):
        self.progresslog( _( "Enable/Disable Repositories" ) )
        if repolist != None:
            for repo in self.repos.repos.values():
                try:
                    if repo.id not in repolist:
                        self.repos.disableRepo( repo.id )
                        #print "Disabled : %s " % repo.id
                    else:
                        self.repos.enableRepo( repo.id )                                               
                        #print "Enabled  : %s " % repo.id
                except yum.Errors.RepoError, e:
                    self.errorlog( 2, str( e ) )
        return True
   
        
    def build_package_lists( self, recentlimit, pkgs ):
        no = 0
        self.progresslog( _( "Building Package Lists : Start") )      
        pkgs.clear()
        #self.progresslog( _( "Building Package Lists : doRpmDBSetup") )      
        self.doRpmDBSetup()
        #self.progresslog( _( "Building Package Lists : doTsSetup") )      
        self.doTsSetup()
        self.progresslog( _( "Building Package Lists : doUpdateSetup") )      
        self.doUpdateSetup()
        self.progresslog( _( "Building Package Lists : Updates" ) )       
        # Get Updates
        uList = pkgs.getPackageList( 'update' )
        if self.conf.obsoletes:
            updates = self.up.getUpdatesList() + self.up.getObsoletesList()
        else:
            updates = self.up.getUpdatesList()
    
        total = len( updates )
        current = 0
        for ( n, a, e, v, r ) in updates:
            current += 1
            matches = self.pkgSack.searchNevra( name=n, arch=a, epoch=e, 
                                               ver=v, rel=r )
            if len( matches ) > 1:
                uList.add( matches[0], True, recentlimit )
                self.log( 4, 'More than one identical match in sack for %s' % matches[0] )
            elif len( matches ) == 1:
                uList.add( matches[0], True, recentlimit )
            else:
                self.log( 4, 'Nothing matches %s.%s %s:%s-%s from update' % ( n, a, e, v, r ) )
            if current % 25 == 0 or current == total:
                self.packageListProgress( current, total )
        self.progresslog( _( "Building Package Lists : %i Updates found" ) % uList.get_lenght() )       
        self.progress.total.next()
        if not self.yumexconf.updateonly:
            rList = pkgs.getPackageList( 'remove' )
            self.progresslog( _( "Building Package Lists : Installed" ) )       
            inst = self.rpmdb.returnPackages()
            total = len( inst )
            current = 0
            for po in inst:
                current += 1
                rList.add( po, False, recentlimit )
                if current % 25 == 0 or current == total:
                    self.packageListProgress( current, total )
            
            self.progresslog ( _( "Building Package Lists : %i Installed packages found" ) %  rList.get_lenght() )     
            self.progress.total.next()
            iList = pkgs.getPackageList( 'install' )
            self.progresslog( _( "Building Package Lists : Available" ) )       
            if self.conf.showdupesfromrepos:
                avail = self.pkgSack.returnPackages()
            else:
                avail = self.pkgSack.returnNewestByNameArch()
    
            total = len( avail )
            current = 0
            for pkg in avail:
                current += 1
                if not self.simpleDBInstalled( pkg.name ): # check name is installed
                    iList.add( pkg, True, recentlimit )
                if current % 50 == 0 or current == total:
                    self.packageListProgress( current, total )
            
            self.progresslog( _( "Building Package Lists : %i Available packages found" ) % iList.get_lenght() )      
            
        self.progresslog( _( "Building Package Lists completed" ) )
        self.progress.total.next()
              
    def packageListProgress( self, current, total ):
        """ Update progress bar for package list building """
        if total != 0:
             frac = float( current )/float( total )
        else:
             frac = 0.0                    
        txt = "%i/%i" % ( current, total )
        self.progress.set_progress( frac, txt )
        self.progress.setTotal( current, total )                       
        self.doGtkEvents()
       
        
        
    def set_state( self, state ):
        self.progresslog( state )
               
    def write_debug_text( self, txt ):
        if self.yumexconf.debug:
            # FIXME: get rid of the oldlogger stuff & use python logging
            self.oldlogger.write_line( txt+"\n", self.oldlogger.style_ps1 )

    def progresslog( self, msg , writeLog=False ):
        self.progress.set_subLabel( msg )
        self.progress.set_progress( 0, " " ) # Blank the progress bar.
        #self.doGtkEvents()
        if writeLog or self.yumexconf.debug:
            self.log( 2, msg )

    def processPkgs( self, pkgs, doAll ):
        self.progresslog( _( "Preparing for install/remove/update" ), True )        
        try:
            # Install
            self.log( 2, _( "--> Preparing for install" ) )
            for po in pkgs['i']:
                tx = self.install( po.pkg )
            # Remove
            self.log( 2, _( "--> Preparing for remove" ) )
            for po in pkgs['r']:
                tx = self.remove( po.pkg )
            # Update
            if doAll:
                self.log( 2, _( "--> Preparing for a full update" ) )
                tx = self.update()
            else:
                self.log( 2, _( "--> Preparing for a partial update" ) )
                if self.yumexconf.fullobsoletion:
                    self.log( 2, _( "--> Adding all obsoletion to transaction" ) )
                    self.doFullObsoletion()
                for po in pkgs['u']:
                    tx = self.update( po.pkg )
            return self.buildTransaction() 
        except yum.Errors.YumBaseError, e:

            return 1, [str( e )]
        #except :
        #   return 1, [_( "Exception was raised" )]
            
        
    def doFullObsoletion( self ):
        # Handle obsoletes:
        if self.conf.obsoletes:
            obsoletes = self.up.getObsoletesTuples( newest=1 )
            for ( obsoleting, installed ) in obsoletes:
                obsoleting_pkg = self.getPackageObject( obsoleting )
                #installed_pkg =  self.rpmdb.packagesByTuple(installed)[0] 
                installed_pkg =  self.rpmdb.searchPkgTuple( installed )[0]                           
                #installed_pkg =  YumInstalledPackage( self.rpmdb.returnHeaderByTuple( installed )[0] )
                self.tsInfo.addObsoleting( obsoleting_pkg, installed_pkg )
                self.tsInfo.addObsoleted( installed_pkg, obsoleting_pkg )

        
    def process_transaction( self ):
        try:
            self.progress.show()
            self.progress.total.next() # -> Download
            self.download_packages()
            self.progress.total.next() # -> Transaction Test
            self.transaction_test()
            self.progress.total.next() # -> Run  Transaction
            self.run_transaction()
            self.progress.hide()
            return True
        except Errors.YumBaseError, e:
            self.last_error = str( e )
            return False
            
    def getDownloadSize( self ):
        
        totsize = 0
        for txmbr in self.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = self.getPackageObject( txmbr.pkgtup )
                if po:
                    size = int( po.size )
                    totsize += size
        return float( totsize )

            
    def download_packages( self ):
        """ Downloading packages and check GPG Signature """
        self.progresslog( _( 'Downloading Packages:' ), True )
        downloadpkgs = []
        for txmbr in self.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = txmbr.po
                if po:
                    downloadpkgs.append( po )
    
        #self.log( 2, 'Downloading Packages:' )
        self.downloadProgress.setStats( self.getDownloadSize() ) 
        try:
            problems = self.downloadPkgs( downloadpkgs ) 
        except: # handle tracebacks error in yum & urlgrapper
            errstring = ''
            errstring += _( 'Error Downloading Packages:\n' )
            for err in sys.exc_info():
                errstring += str( err )
                raise yum.Errors.YumBaseError( errstring )
            
        self.downloadProgress.clearStats()
        if len( problems.keys() ) > 0:
            errstring = ''
            errstring += _( 'Error Downloading Packages:\n' )
            for key in problems.keys():
                errors = yum.misc.unique( problems[key] )
                for error in errors:
                    errstring += '  %s: %s\n' % ( key, error )
                raise yum.Errors.YumBaseError( errstring )
                  # Check GPG signatures
        self.progresslog( _( 'Checking GPG Signatures:' ), True )
        try:
            self.gpgsigcheck( downloadpkgs )
        except yum.Errors.YumBaseError, errmsg:
            errstring = ''
            errstring += _( 'Error checking package signatures:\n' )
            errstring += str(errmsg)
            raise yum.Errors.YumBaseError( errstring )
            
        except: # handle tracebacks error in yum & urlgrapper
            errstring = ''
            errstring += _( 'Traceback in checking package signatures:\n' )
            for err in sys.exc_info():
                errstring += str( err )
                raise yum.Errors.YumBaseError( errstring )

    def transaction_test( self ):
        """ Do Traaction Test"""
        #self.log( 2, 'Running Transaction Test' )
        self.progresslog( _( 'Running Transaction Test' ), True )
        tsConf = {}
        for feature in ['diskspacecheck']: # more to come, I'm sure
            tsConf[feature] = getattr( self.conf, feature )
        #
        testcb = yumexRPMCallback( progress=self.progress )
        testcb.tsInfo = self.tsInfo
        # clean out the ts b/c we have to give it new paths to the rpms 
        del self.ts
  
        self.initActionTs()
        # save our dsCallback out
        dscb = self.dsCallback
        self.dsCallback = None # dumb, dumb dumb dumb!
        self.populateTs( keepold=0 ) # sigh
        tserrors = self.ts.test( testcb, conf=tsConf )
        del testcb
  
        #self.log( 2, 'Finished Transaction Test' )
        self.progresslog( _( 'Finished Transaction Test' ), True )
        if len( tserrors ) > 0:
            errstring = _( 'Transaction Check Error: ' )
            for descr in tserrors:
                 errstring += '  %s\n' % descr 
            raise yum.Errors.YumBaseError( errstring ) 

        #self.log( 2, 'Transaction Test Succeeded' )
        self.progresslog( _( 'Transaction Test Succeeded' ), True )
        del self.ts
        # put back our depcheck callback
        self.dsCallback = dscb
        

    def run_transaction( self ):
        self.initActionTs() # make a new, blank ts to populate
        self.populateTs( keepold=0 ) # populate the ts
        self.ts.check() #required for ordering
        self.ts.order() # order
        output = 1
        cb = yumexRPMCallback( progress=self.progress, filelog=self.filelog )
        cb.filelog = self.filelog # needed for log file output
        cb.tsInfo = self.tsInfo

        #self.log( 2, 'Running Transaction' )
        self.progresslog( _( 'Running Transaction' ), True )
        self.runTransaction( cb=cb )

        # close things
        self.log( 1, self.postTransactionOutput() )
      
      
    def setupFileLog( self, logfile='/var/log/yum.log' ):  
        if self.conf.uid == 0:
            try:
                filelogger = logging.getLogger( "yumex.filelogging" )
                filehandler = logging.FileHandler( logfile )
                formatter = logging.Formatter( "%(asctime)s %(message)s", 
                    "%b %d %H:%M:%S" )
                filehandler.setFormatter( formatter )
                filelogger.addHandler( filehandler )
                filelogger.propagate = False
                
            except Exception, e:
                logging.getLogger( "yumex" ).critical( 'Cannot open logfile %s', logfile )


    def printtime( self ):
        """ Overloading printtime in YumOutput Class"""
        return time.strftime( '%b %d %H:%M:%S (yumex) ', time.localtime( time.time() ) )

        
    def doGtkEvents( self ):
        while gtk.events_pending():      # process gtk events
            gtk.main_iteration()

# ---------------------------------------------------------------------------
# methods copied from cli.py (yum)
# --------------------------------------------------------------------------

# Confirmation check uncommented and strings are made translatable.

    def gpgsigcheck(self, pkgs):
        '''Perform GPG signature verification on the given packages, installing
        keys if possible

        Returns non-zero if execution should stop (user abort).
        Will raise YumBaseError if there's a problem
        '''
        for po in pkgs:
            result, errmsg = self.sigCheckPkg(po)

            if result == 0:
                # Verified ok, or verify not req'd
                continue            

            elif result == 1:
               #if not sys.stdin.isatty() and not self.conf.assumeyes:
               #   raise yum.Errors.YumBaseError, \
               #         'Refusing to automatically import keys when running ' \
               #         'unattended.\nUse "-y" to override.'

               # the callback here expects to be able to take options which
               # userconfirm really doesn't... so fake it
               self.getKeyForPackage(po, lambda x, y, z: self.userconfirm())

            else:
                # Fatal error
                raise yum.Errors.YumBaseError, errmsg

        return 0

 
# default to self.extcmds commented and strings made translatable
    def installGroups( self, grouplist=None ):
        """for each group requested do 'selectGroup' on them."""
        
        self.doRepoSetup()
        pkgs_used = []
        
#        if grouplist is None:
#            grouplist = self.extcmds
        
        for group_string in grouplist:
            group = self.comps.return_group( group_string )
            if not group:
                self.errorlog( 0, _( 'Warning: Group %s does not exist.' ) % group )
                continue
            
            try:
                txmbrs = self.selectGroup( group.groupid )
            except yum.Errors.GroupsError, e:
                self.errorlog( 0, _( 'Warning: Group %s does not exist.' ) % group )
                continue
            else:
                pkgs_used.extend( txmbrs )
            
        if not pkgs_used:
            return 0, [_( 'No packages in any requested group available to install or update' )]
        else:
            return 2, [_( '%d Package(s) to Install' ) % len( pkgs_used )]

# default to self.extcmds commented and strings made translatable
    def removeGroups( self, grouplist=None ):
        """Remove only packages of the named group(s). Do not recurse."""

        pkgs_used = []
        
#        if grouplist is None:
#            grouplist = self.extcmds
        
        erasesbygroup = []
        for group_string in grouplist:
            try:
                txmbrs = self.groupRemove( group_string )
            except yum.Errors.GroupsError, e:
                self.errorlog( 0, _( 'No group named %s exists' ) % group_string )
                continue
            else:
                pkgs_used.extend( txmbrs )
                
        if not pkgs_used:
            return 0, [_( 'No packages to remove from groups' )]
        else:
            return 2, [_( '%d Package(s) to remove' ) % len( pkgs_used )]

          
class yumexRepoCache:
    """ This class contains info about urls for already load repos """
    
    def __init__( self, recentlimit ):
        self.parser = ConfigParser.ConfigParser()
        self.file = '/var/cache/yum/yumex-mirror-cache.conf'       
        self.cache = yumexUtils.mirrorCache( self.parser, self.file )
        self.recentlimit = recentlimit
        self.updated = False
    
    def clear( self ):
        """ Clear the cache and create a new one. """
        del self.parser
        del self.cache
        self.parser = ConfigParser.ConfigParser()
        self.cache = yumexUtils.mirrorCache( self.parser, self.file )
        self.updated = False
        try:
            os.unlink( self.file )
        except OSError, e:
            print _( '%s was not deleted' ) % self.file
    
    def get_repo_names( self ):        
        return self.cache.get_names()
        
    def add( self, name ):
        self.cache.add( name )
            
    def set_urls( self, name, urls, method ):
        rc = self.cache.get( name )
        if rc:
            rc.urls = urls
            rc.lastscan = time.time()
            rc.detection = method
            self.updated = True
            return True
        else:
            return False

    def get_urls( self, name ):
        rc = self.cache.get( name )
        if rc:
            return rc.urls
        else:
            return None
            
    def has_item( self, name, method ):
        state = False
        rc = self.cache.get( name )
        if rc:
            if time.time() < rc.lastscan + self.recentlimit:
                if method == rc.detection:
                    state = True
        return state
    
    def load( self ):
        self.cache.load()
             
        
    def save( self ):
        self.cache.save()




                     
class yumexAction:
    """ This class controls installing,updating,removing """    

    def __init__( self, parent, xml, dlg, console, status ):
        self.parent = parent
        self.progress = dlg
        self.status = status
        self.selected = None
        self.console = console
        self.nonselected = None
        self.yumexbase = None
        self.xml = xml
        
    def set_yumbase( self, yumexbase ):
        self.yumexbase = yumexbase
        
    def set_selected( self, sel, nonsel = None ):
        self.selected = sel
        self.nonselected = nonsel
        
    def set_repos( self, enabled, disabled ):
        self.repoEnabled = enabled
        self.repoDisabled = disabled
              
    def process_package_queue( self, pkgs, doAll, doSafe=False ):
        """ Workflow for processing package queue """
        #(pkgs,doAll) = args
        self.set_status( _( "Preparing for install/remove/update" ) )
        self.progress.total.setup( const.PACKAGE_PROGRESS_STEPS )        
        total = len( pkgs['i'] )+len( pkgs['u'] )+len( pkgs['r'] )
        state = True
        quit = False
        if total > 0:
            self.progress.show()
            self.progress.set_mainLabel( _( "Processing Packages in queue" ) )
            if doSafe:
                rc, msg = self.process_safe( pkgs )
            else:
                rc, msg = self.yumexbase.processPkgs( pkgs, doAll )        
            if rc != 2:
                self.yumexbase.write_debug_text( self.yumexbase.listTransaction() )        
                m = self.get_str( msg )
                msg = _( "Error in Dependency Resolution" )
                rc = yumexDialogs.err_box( self.parent, _( "Error" ), msg, m )
                state=False
            else:
                if self.get_confimation( "" ):
                    quit = self.process_packages()
                    if quit == None:
                        state = False
                else:
                    state = False
            self.progress.hide()
            if quit:
                return "QUIT"
            return state
        else:
            self.set_status( _( "No packages selected" ) )
            return state

    def process_safe( self, pkgs ):
        goodlist={'u':[], 'i':[], 'r':[]}
        badlist= {'u':[], 'i':[], 'r':[]}
        for action in pkgs.keys():
            pkglist = pkgs[action]
            for pkg in pkglist:
                if self.process_one( pkg, action ):
                    list = goodlist[action]
                    list.append( pkg )
                    goodlist[action] = list
                else:
                    list = badlist[action]
                    list.append( pkg )
                    badlist[action] = list
                #print len(self.yumexbase.tsInfo)
        # Check if the transaction is ok
        self.bad = badlist
        for action in badlist.keys():
            for po in badlist[action]:
                self.yumexbase.log( 2, '--> Skipping %s, dependencies cant be resolved' % str( po ) )
        return self.yumexbase.processPkgs( goodlist, False )        
        
            

    def process_one( self, pkg, action ):
        try:
            self.yumexbase.reset_ts()
            if action == 'i':
                tx = self.yumexbase.install( pkg.pkg )
            elif action == 'r':
                tx = self.yumexbase.remove( pkg.pkg )
            else:
                tx = self.yumexbase.update( pkg.pkg )
            self.yumexbase.log( 2, "Checking : %s " % str( pkg ) )
            rc, msg = self.yumexbase.buildTransaction()
            if rc != 2:
    #            self.yumexbase.tsInfo.remove(pkg.pkgtup)
                self.yumexbase.log( 2, "Skipping : %s " % str( pkg ) )
                return False
            else:
                return True
        except yum.Errors.YumBaseError, e:
            self.yumexbase.log( 2, "Error : %s " % str( e ) )            
            self.yumexbase.log( 2, "Skipping : %s " % str( pkg ) )
            return False
            
        

    def install_groups( self, grps ):
        self.progress.total.setup( const.PACKAGE_PROGRESS_STEPS )        
        self.set_status( _( "Preparing for install Groups" ) )
        state = True
        self.progress.show()
        self.progress.set_mainLabel( _( "Installing Groups" ) )
        rc, msg = self.yumexbase.installGroups( grps )
        if rc != 2:
            self.yumexbase.write_debug_text( self.yumexbase.listTransaction() )        
            m = self.get_str( msg )
            self.yumexbase.write_debug_text( m )                   
            msg = _( "Error in preparation for package installing" )
            rc = yumexDialogs.err_box( self.parent, _( "Error" ), msg, m )
            state = False
        else:
            if self.get_confimation( _( "Install" ) ):
                if not self.process_packages():
                    state=False
                else:
                    if self.yumexbase.yumexconf.autocleanup:
                        rc, msg = self.yumexbase.cleanPackages()
                        self.yumexbase.log( 2, msg[0] )
            else:
                state = False
        self.progress.hide()
        return state

    def remove_groups( self, grps ):
        self.progress.total.setup( const.PACKAGE_PROGRESS_STEPS )        
        self.set_status( _( "Preparing for Removing Groups" ) )
        state = True
        self.progress.show()
        self.progress.set_mainLabel( _( "Removeing Groups" ) )
        rc, msg = self.yumexbase.removeGroups( grps )
        if rc != 2:
            self.yumexbase.write_debug_text( self.yumexbase.listTransaction() )        
            m = self.get_str( msg )
            self.yumexbase.write_debug_text( m )                   
            msg = _( "Error in preparation for package removing" )
            rc = yumexDialogs.err_box( self.parent, _( "Error" ), msg, m )
            state = False
        else:
            if self.get_confimation( _( "Remove" ) ):
                if not self.process_packages():
                    state=False
            else:
                state = False
        self.progress.hide()
        return state

            
    def get_str( self, lst ):
        rc = ""
        for l in lst:
            rc = rc + l+"\n"
        return rc
            
    
    def set_status( self, text ):
        context_id = self.status.get_context_id( "Status" )
        self.status.push( context_id, text )
    
    def get_confimation( self, task ):
        self.yumexbase.write_debug_text( self.yumexbase.listTransaction() )
        self.yumexbase.tsInfo.makelists()
        self.progress.hide( False )
        pkg = self.make_pkgs_list()
        size = self.get_download_size()
        rc = yumexDialogs.confimation_dialog( self.parent, self.xml, task, pkg, size )
        self.progress.show
        return rc
        
    def make_pkgs_list( self ):
        list = []
        sublist = []
        for ( action, pkglist ) in [( _( 'Installing' ), self.yumexbase.tsInfo.installed ), 
                            ( _( 'Updating' ), self.yumexbase.tsInfo.updated ), 
                            ( _( 'Removing' ), self.yumexbase.tsInfo.removed ), 
                            ( _( 'Installing for dependencies' ), self.yumexbase.tsInfo.depinstalled ), 
                            ( _( 'Updating for dependencies' ), self.yumexbase.tsInfo.depupdated ), 
                            ( _( 'Removing for dependencies' ), self.yumexbase.tsInfo.depremoved )]:
            for txmbr in pkglist:
                ( n, a, e, v, r ) = txmbr.pkgtup
                evr = txmbr.po.printVer()
                repoid = txmbr.repoid
                pkgsize = float( txmbr.po.size )
                size = self.yumexbase.format_number( pkgsize )
                alist=[]
                for ( obspo, relationship ) in txmbr.relatedto:
                    if relationship == 'obsoletes':
                        appended = 'replacing  %s.%s %s' % ( obspo.name, 
                            obspo.arch, obspo.printVer() )
                        alist.append( appended )
                el = ( n, a, evr, repoid, size, alist )
                sublist.append( el )
            if pkglist:
                list.append( [action, sublist] )
                sublist = []
        return list        

    def get_package_list( self, List ):
        """ Build package list for confimation dialog"""
        txt = []
        if List == []: return ""
        for mbr in List:
            pkg_size = self.yumexbase.format_number( float( mbr.po.size ) )
            ver = mbr.po.printVer()
            el = ( mbr.name, ver, pkg_size )
            txt.append( el )
        return txt

    def get_download_size( self , asFloat=False ):
        
        totsize = 0
        for txmbr in self.yumexbase.tsInfo.getMembers():
            if txmbr.ts_state in ['i', 'u']:
                po = self.yumexbase.getPackageObject( txmbr.pkgtup )
                if po:
                    size = int( po.size )
                    totsize += size
        if asFloat:
            return  float( totsize )
        else:
            return self.yumexbase.format_number( float( totsize ) )
        
    def debug_out( self, func, txt ):
        self.console.write_line( "DEBUG:(%s) - %s \n" % ( func, txt ), self.console.style_ps1 ) 
    

    def debug_showrepos( self ):
        enabled = ""
        for r in self.repoEnabled:
            enabled = enabled + "    "+r + '\n'
        self.console.write_line( _( "Enabled Repositories:\n%s" ) % enabled, self.console.style_ps1 ) 
             
                    
    def process_packages( self ):
        self.set_status( _( "Processing packages (See output for details)" ) )
        if not self.yumexbase.process_transaction():            
            err = "\n%s" % self.yumexbase.last_error
            rc = yumexDialogs.err_box( self.parent, _( "Error" ), _( "Error in Transaction" ), err )            
            return None
        else:
            rc = yumexDialogs.msg_box( self.parent, _( "Packages Processing" ), _( "Packages Processing completed ok" ) )
            #yumexDialogs.send_notification(_("Packages Processing completed ok"))
            return rc
           