#!/usr/bin/python -tt
# -*- coding: iso-8859-1 -*-
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# $Id: yumexConst.py,v 1.14 2006/02/15 12:41:42 timlau Exp $

# This files contains Yum Extender constants

import os
import sys
from yumex.yumexi18n import _


__yumex_version__   = "1.2.2"

YUM_PID_FILE = '/var/run/yum.pid'

MAIN_PATH = os.path.abspath( os.path.dirname( sys.argv[0] ) );
GLADE_FILE = MAIN_PATH+'/yumex.glade'  

if MAIN_PATH == '/usr/share/yumex':    
    PIXMAPS_PATH = '/usr/share/pixmaps/yumex'
else:
    PIXMAPS_PATH = MAIN_PATH+'/../gfx'

# Make mirrordetection type output translatable (Console Output only)

MIRROR_DETECTION_TYPES = { 'default' : _( 'default' ) , 
                           'fastest' : _( 'fastest' ) , 
                           'best'    : _( 'best' )     }

                           

# Time i seconds before mirrorcache is to old
DAY_IN_SECONDS = 86400
MIRROR_RECENT_LIMIT = 7 * DAY_IN_SECONDS


PROCESS_ACTION_TYPES = { 'i' : _( 'installation' ) , 
                         'u' : _( 'updating' ) , 
                         'r' : _( 'removal' )}

PROGRESS_ACTION_TYPES = { 'i' : _( 'Install' ) , 
                          'u' : _( 'Update' ) , 
                          'r' : _( 'Remove' )}

# YUM Config, Repo Setup
REPOSETUP_PROGRESS_STEPS = ( 0.1, # Yum Config
                            0.2, # Repo Setup
                            0.1, # Updates List   
                            0.2, # Install List   
                            0.4 ) # Availible List

PACKAGE_PROGRESS_STEPS = ( 0.1, # Depsolve
                           0.5, # Download
                           0.1, # Transaction Test
                           0.3 ) # Running Transaction

PACKAGE_PAGE_LIST = ['install', 'update', 'remove']

# Categories in the Category view
# Text, getter, attribute, sort
#
# Textl     : The text shown in the Combobox.
# getter    : The function to get the data (YumexPackageStore)
# attribute : The attribute to get from the package object.
# sort      : Sort the keys before adding to view.

PACKAGE_CATEGORIES = { 
'0' : ( _( 'RPM Groups' ), 'getByAttr', 'group', True ), 
'1' : ( _( 'Repositories' ), 'getByProperty', 'repoid', True ), 
'2' : ( _( 'Architecture' ), 'getByProperty', 'arch', True ), 
'3' : ( _( 'Sizes' ), 'getBySizes', '', False ), 
'4' : ( _( 'Age' ), 'getByAge', '', False )}

PACKAGE_PAGE_HEADERS = {
'install': _( "<span size=\"xx-large\" color=\"black\">Install Packages</span>" ), 
'remove' : _( "<span size=\"xx-large\" color=\"black\">Remove Packages</span>" ), 
'category' : _( "<span size=\"xx-large\" color=\"black\">Packages by category</span>" ), 
'update' : _( "<span size=\"xx-large\" color=\"black\">Update Packages</span>" )}

PACKAGE_PAGE_IMAGES = {
'install': PIXMAPS_PATH + '/button-install.png' , 
'remove' : PIXMAPS_PATH + '/button-remove.png', 
'category' : PIXMAPS_PATH + '/button-group.png', 
'update' : PIXMAPS_PATH + '/button-update.png'}

NOTEBOOK_PAGES = {
'update'  : 1, 
'remove'  : 1, 
'install' : 1, 
'category': 1, 
'repos'   : 0, 
'output'  : 2, 
'group'   : 3, 
'queue'   : 4 }

PAGE_BUTTONS = {
'update'  : 'toolButton0', 
'install' : 'toolButton1', 
'remove'  : 'toolButton2', 
'category' : 'toolButton3', 
'group'   : 'toolButton4', 
'queue'   : 'toolButton5', 
'repos'   : 'toolButton6', 
'output'  : 'toolButton7'}

__credits__ = """
Coding:
    Tim Lauridsen
    Dawid Zamirski

Translators:
  MATSUURA Takanori             (Japanese)
  Rodrigo Padula de Oliveira    (Brazilian)
  Eric Tanguy                   (French)
  Soohyung Cho                  (Korean)
  Danilo                        (Italian)
  Serta . YILDIZ              (Turkish)
  Dawid Zamirski                (Polish)
  Tero Hietanen                 (Finnish)
  Dieter Komendera              (German)
  Tim Lauridsen                 (Danish)
  Maxim Dziumanenko             (Ukrainian)
  Novotny Lukas                 (Czech)
  Szll Tams                   (Hungarian)
  Leonid Kanter                 (Russian)   
  Diego Alonso                 (Spanish)   
  A Singh Alam                 (Punjabi)    
  
Thanks to:
  The Yum developers
  The yumex translators  
  Hrishikesh Ballal 
  Nicu Buculei
  Dennis Cranston 
  All yumex users 
"""
