#!/usr/bin/python -tt
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# $Id: yumexPackages.py,v 1.8 2006/02/14 11:02:33 timlau Exp $

#import yumexBase
import re
#import yumexUtils
import time
import gtk
import types
import gobject

from yumgui import format_number
from yumgui.packages import PackageWrapper

class yumexPackage( PackageWrapper ):
    """ This class contains a yumPackage and some extra features used by
    yumex """
    
    def __init__( self, pkg, recentlimit, avail=True ):
        PackageWrapper.__init__( self, pkg, avail )
        self.selected = False
        self.visible = True
        self.queued = None  
        self.time = self._get_time()
        if self.time > recentlimit:
            self.recent = True
        else:
            self.recent = False
 
    def set_select( self, state ):
        self.selected = state

    def set_visible( self, state ):
        self.visible = state

        
class yumexPackageList:
    """ This class is a list of yumexPackages """
    
    def __init__( self ):
        self.list  = []
        #self.list = gtk.ListStore(object)
        self.filter = []
        self.arch_filter = None
        
    def clean( self ):
        del self.list
        self.list = []
        self.filter = []
        self.arch_filter = None       

    def dump( self ):
        for el in self.list:
            pkg = el[0]
            print pkg.name, pkg.visible

    def add( self, pkg, avail, recentlimit ):
        #print " Name : %s avail : %s" % (pkg.name,avail)        
        newpkg = yumexPackage( pkg, recentlimit, avail )
        self.list.append( newpkg )
        
    def add_list( self, pkglist ):
        for pkg in pkglist:
            #print pkg
            self.list.append( pkg )
            
    def getList( self ):
        return self.list
        
    def set_selection( self, name, ver, arch, state ):
        pkg = self.find_pkg( name, ver, arch )
        if pkg != None:
            if state:
                pkg.select()
            else:
                pkg.unselect()
        
            
    def find_pkg( self, name, ver, arch ):
        rc = None
        for pkg in self.list:
            #print pkg
            if pkg.name == name:
                v = pkg.ver
                if pkg.arch == arch and v == ver:
                    rc = pkg
                    break
        return rc
    
    def find_po_by_name( self, n, a ):
        '''  Find package by name,arch and return yum po'''
        rc = None
        for pkg in self.list:
            if n == pkg.name:
                if a == pkg.arch:
                    rc = pkg.pkg
                    break
        return rc

    def find_pkg_by_name( self, n, a ):
        '''  Find package by name,arch and return yumexPackage'''
        rc = None
        for pkg in self.list:
            if n == pkg.name:
                if a == pkg.arch:
                    rc = pkg
                    break
        return rc

    def find_pkg_by_nva( self, n, v, a ):
        '''  Find package by name,arch and return yumexPackage'''
        rc = None
        for pkg in self.list:
            if n <> pkg.name:
                continue
            if a <> pkg.arch:
                continue
            if v <> pkg.ver:
                continue
            return pkg
        return rc
    
    def find_ver( self, name, arch ):
        rc = None
        for pkg in self.list:
            if pkg.name == name:
                if pkg.arch == arch:
                    rc = pkg.ver
                    #print "found : %s" % rc
        return rc
    
    def get_active_list( self ):
        return self.list

    def get_selected( self ):
        return [pkg for pkg in self.list if pkg.selected]

    def get_nonselected( self ):
        nonselected = []
        for pkg in self.list:
            if not pkg.selected:
                nonselected.append( pkg.pkg )
        return nonselected
        
    def get_lenght( self ):
        return len( self.list )
        
    def select_by_fullname( self, keys ):    
        for pkg in self.list:
            if pkg.get_fullname() in keys:
                pkg.set_select( True )      
        
    def select_all( self ):
        for pkg in self.list:
            if pkg.visible:
                if not pkg.selected:
                    pkg.set_select( True )      

    def deselect_all( self ):
        for pkg in self.list:
            if pkg.visible:
                if pkg.selected:
                    pkg.set_select( False )      
        
        
    def apply_filter( self, filter, recentFlag=0, selectFlag=None ):
        """
        Apply filters to packages in list.
        All packages there matches the filter will have
        visible=True, else they will have visible=False
        only packages with visible=True, will be shown in the
        Install/Update/Remove TreeViews.
        """
        if filter != None:
            self.filter = filter.split( ' ' )
            self.arch_filter = self._get_arch_filter( filter )
            self.repo_filter = self._get_repo_filter( filter )
        for pkg in self.list:
            if selectFlag and not pkg.selected:
                pkg.set_visible( False )
            elif recentFlag == 1 and not pkg.recent:
                pkg.set_visible( False )
            elif recentFlag == 2 and not pkg.queued:
                pkg.set_visible( False )
            else:
                pkg.set_visible( True )
                if len( self.filter ) != 0:
                    self._check_filter( pkg )
                                
    def _check_filter( self, pkg ):
        exit = self._check_special_filters( pkg )
        if exit:
            return
        name = pkg.name
        summary = pkg.summary
        desc = pkg.description
        if name == None: name = ""
        if summary == None: summary = ""
        if desc == None: desc = ""
        keys = [name.upper(), summary.upper(), desc.upper()]
        words = self.filter
        # Check for filter words
        for word in words:
            if word.find( "=" ) <> -1:
                pass
            else:
                if not self._check_for_word( word, keys ):
                    pkg.set_visible( False )
                    return
                
    def _check_special_filters( self, pkg ):
        exit = False
        # Check for Architechture filter
        if pkg.arch == None:
            arch = ""
        else:
            arch = pkg.arch.upper()
        if self.arch_filter != None:
            if arch == self.arch_filter:
                pkg.set_visible( True )
            else:
                pkg.set_visible( False )
                exit = True
                return exit
        # Check for Repository Filter
        repo = pkg.repoid        
        if self.repo_filter != None and repo != "installed":
            if repo.upper() == self.repo_filter:
                pkg.set_visible( True )
            else:
                pkg.set_visible( False )
                exit = True
        return exit
        
                
    def _get_arch_filter( self, flt ):
        """ Check for ARCH=<Actitecture> in filter words """
        arch_re = re.compile( '(?<=ARCH=)\w+' )
        arch_rc = arch_re.search( flt )
        if arch_rc != None:
            #print "Arch : %s" % arch_rc.group( 0 )
            arch_flt = arch_rc.group( 0 )
        else:
            arch_flt = None
        return arch_flt

    def _get_repo_filter( self, flt ):
        """ Check for REPO=<Actitecture> in filter words """
        repo_re = re.compile( '(?<=REPO=)\w+' )
        repo_rc = repo_re.search( flt )
        if repo_rc != None:
            #print "Repo : %s" % repo_rc.group( 0 )
            repo_flt = repo_rc.group( 0 )
        else:
            repo_flt = None
        return repo_flt

            
    def _check_for_word( self, word, keys ):
        for key in keys:
            if key.find( word ) <> -1:
                return True
        return False
        

        
class YumexPackageStorage:
    def __init__( self ):
        self.pkgs = { 'install' : yumexPackageList(), 
                      'update' : yumexPackageList(), 
                      'remove' : yumexPackageList()}          
    
    def add( self, type, pkg, avail, recentlimit ):
        self.pkgs[type].add( pkg, avail, recentlimit )
        
    def getPackageList( self, type ):
        return self.pkgs[type]
            
    def clear( self ):
        del self.pkgs
        self.pkgs = { 'install' : yumexPackageList(), 
                      'update' : yumexPackageList(), 
                      'remove' : yumexPackageList()}          
        
    def getByProperty( self, type, category ):
        list = self.pkgs[type].get_active_list()
        dict = {}
        for pkg in list:
            val = getattr( pkg, category )
            # Strip newline in keys
            if val:
                val = val.strip( '\n' )
            if dict.has_key( val ):
                dict[val].append( pkg )
            else:
                dict[val] = [pkg]
        return dict, dict.keys()
 
    def getByAttr( self, type, attr ):
        list = self.pkgs[type].get_active_list()
        dict = {}
        for pkg in list:
            val = pkg.getAttr( attr )
            # Strip newline in keys
            if val:
                val = val.strip( '\n' )
            if dict.has_key( val ):
                dict[val].append( pkg )
            else:
                dict[val] = [pkg]
        return dict, dict.keys()
        
    def getBySizes( self, type, attr ):
        list = self.pkgs[type].get_active_list()
        keys = [
            '0 KB - 100 KB', 
            '100 KB - 1 MB', 
            '1 MB - 10 MB' , 
            '10 MB - 50 MB', 
            '50+ MB']
        
        dict = {}
        for pkg in list:
            val = self._getSizeKey( pkg.size )
            if dict.has_key( val ):
                dict[val].append( pkg )
            else:
                dict[val] = [pkg]
        return dict , keys       
        
    def _getSizeKey( self, size ):
        kb = 1024
        mb = 1024*1024
        if size < 100*kb:
            return '0 KB - 100 KB'
        elif size < mb:
            return '100 KB - 1 MB'
        elif size < 10*mb:
            return '1 MB - 10 MB' 
        elif size < 50*mb:
            return '10 MB - 50 MB'
        else:
            return '50+ MB'

    def getByAge( self, type, attr ):
        list = self.pkgs[type].get_active_list()
        keys = [
            '0 - 7 Days', 
            '7 - 14 Days', 
            '14 - 21 Days', 
            '21  - 30 days', 
            '30 - 90 days', 
            '90+ days']
        dict = {}
        for pkg in list:
            val = self._getAgeKey( pkg._get_time() )
            if dict.has_key( val ):
                dict[val].append( pkg )
            else:
                dict[val] = [pkg]
        return dict , keys       

            
    def _getAgeKey( self, date ):
        now = time.time()
        days = 86400 # Seconds
        if date > now-7*days:
            return '0 - 7 Days'
        elif date > now-14*days:
            return '7 - 14 Days'
        elif date > now-21*days:
            return '14 - 21 Days'
        elif date > now-30*days:
            return '21  - 30 days'
        elif date > now-90*days:
            return '30 - 90 days'
        else:
            return '90+ days'
            
            
            
           